# --------------------------------------------------------------------------
# This code is adapted from dht11_lcd5.py
# https://gitlab.com/IOTGarage/iot-lab-book/-/blob/070cf41b33cffdc1f1f90f6a42e1e16215db24be/LAB%2002%20-%20Single-board%20Computer%20Programming/dht11_lcd5.py
# I have modified the code to take RGB values and text input from arguments rather than from the dht11 sensor.
# I also added error handling for the RGB values and text input, such as clamping the RGB values to the range of 0-255 and checking if the text input is empty.
# This means the code can be used to display any text and RGB values on the LCD based on parameters passed by Node-Red, with appropriate debugging capabilities.
# --------------------------------------------------------------------------

#!/usr/bin/env python3
import time
import sys
import smbus2 as smbus
import RPi.GPIO as GPIO

# Determine the Raspberry Pi revision and assign the correct I2C bus.
rev = GPIO.RPI_REVISION
if rev == 2 or rev == 3:
    bus = smbus.SMBus(1)   # Newer Pi boards use I2C bus 1
else:
    bus = smbus.SMBus(0)   # Older boards might use I2C bus 0

# I2C addresses for the Grove LCD
DISPLAY_RGB_ADDR = 0x30
DISPLAY_TEXT_ADDR = 0x3e

def setRGB(r, g, b):
    # Set the RGB color of the display
    bus.write_byte_data(DISPLAY_RGB_ADDR, 0x04, 0x15)
    bus.write_byte_data(DISPLAY_RGB_ADDR, 0x06, r)
    bus.write_byte_data(DISPLAY_RGB_ADDR, 0x07, g)
    bus.write_byte_data(DISPLAY_RGB_ADDR, 0x08, b)

# Function to send commands to the LCD display - clear display, set line, etc.
def textCommand(cmd):
    bus.write_byte_data(DISPLAY_TEXT_ADDR, 0x80, cmd)

# Function to send text to the LCD display - 16 characters per line, 2 lines total
# The function will handle newlines and line breaks automatically
def setText(text):
    if not text.strip():
        print("Error: Text input cannot be empty.")
        sys.exit(1)
    # Clear display
    textCommand(0x01)
    time.sleep(0.05)
    # Display on, no cursor
    textCommand(0x08 | 0x04)
    # 2-line display mode
    textCommand(0x28)
    time.sleep(0.05)

    count = 0
    row = 0

    # For each character in the text, write it to the display
    # If a newline character is encountered, move to the next line
    # If 16 characters are written, move to the next line
    for c in text:
        if c == '\n' or count == 16:
            count = 0
            row += 1
            # If we already moved to the second line, stop writing further text
            if row == 2:
                break
            # Command 0xc0 moves cursor to the beginning of the second line
            textCommand(0xc0)
            if c == '\n':
                continue
        count += 1
        bus.write_byte_data(DISPLAY_TEXT_ADDR, 0x40, ord(c))

if __name__ == "__main__":
    # Check if the correct number of arguments are provided
    # The script expects 4 arguments, the first argument is the script name, so we check for 5 arguments in total
    if len(sys.argv) != 5:
        print("Usage: lcd_display.py <red> <green> <blue> <text>")
        sys.exit(1)

    try:
        # Read RGB values from command line arguments
        r = int(sys.argv[1])
        g = int(sys.argv[2])
        b = int(sys.argv[3])
        
        # Check if the RGB values are within the valid range (0-255)
        if r < 0 or r > 255:
            print(f"Warning: Red value {r} out of range. Clamping to valid range.")
        if g < 0 or g > 255:
            print(f"Warning: Green value {g} out of range. Clamping to valid range.")
        if b < 0 or b > 255:
            print(f"Warning: Blue value {b} out of range. Clamping to valid range.")
        
        # If not valid, clamp them to the valid range and print a warning message
        r = max(0, min(255, r))
        g = max(0, min(255, g))
        b = max(0, min(255, b))

    # Handle non-integer RGB inputs
    # If the input is not an integer, set to default value (white) and print a warning message
    except ValueError:
        print("Error: RGB inputs must be integers. Using substitute colour (white).")
        r, g, b = 255, 255, 255

    text = sys.argv[4]

    setRGB(r, g, b)
    setText(text)
